﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ClientApp.DSWSReference;

namespace ClientApp
{
    public class ServiceHelper
    {
        private string token = string.Empty;

        public string GetToken()
        {
            if (!string.IsNullOrEmpty(token))
                return token;

            // Create a token request
            var tokenRequest = new DSGetTokenRequest()
            {
                UserName = "", //add your Datastream credentials here
                Password = ""
            };

            string tokenValue = null;
            DateTime tokenExpiry = default(DateTime);
            // Instantiate a client object to make the request
            using (var dsClient = new DSServiceClient())
            {
                // Issue a call to get the token
                try
                {
                    var tokenResponse = dsClient.GetToken(tokenRequest);

                    // Read the token from the response
                    tokenValue = tokenResponse.TokenValue;
                    tokenExpiry = tokenResponse.TokenExpiry;
                }
                catch (Exception e)
                {
                    // Something went wrong with our request
                    // process the error message in e.Message

                    Console.WriteLine("Could not retrieve token. Service returned an error: " + e.Message );                    
                    return null;           
                }
            }
            
            this.token = tokenValue;

            Console.WriteLine("Token retrieved successfully!");
            Console.ReadLine();

            return tokenValue;
        }

        public void GetData()
        {
            var request = new DSGetDataRequest()
            {
                TokenValue = this.token,
                DataRequest = new DSDataRequest()
                {
                    Instrument = new DSInstrument() { Value = "VOD" },
                    DataTypes = new[] { new DSDataType() { Value = "PL" } },
                    Date = new DSDate()
                    {
                        Kind = DSDateKind.TimeSeries,
                        Start = "-30D",
                        End = "-10D",
                        Frequency = DSDateFrequencyNames.D.ToString()
                    }
                }
            };

            // Get the data
            DSDataResponse response = null;

            try
            {
                using (var dsclient = new DSServiceClient())
                    response = dsclient.GetData(request).DataResponse;
            }
            catch (Exception e)
            {
                // Something went wrong with our request
                // process the error message in e.Message

                Console.WriteLine("An error occurred while retrieving data. Service returned an error: " + e.Message);
                Console.ReadLine();
            }

            // Process the response
            foreach (var datatypeValue in response.DataTypeValues)
            {
                // This is the datatype requested
                string requestedDataType = datatypeValue.DataType;
                
                // Access the symbols for the datatype
                // If the requested instrument was a list, there will be multiple symbols
                foreach (var symbolVal in datatypeValue.SymbolValues)
                {
                    string symbol = symbolVal.Symbol;

                    Console.Write(Environment.NewLine);
                    Console.WriteLine(symbol + ":" + requestedDataType);

                    if (symbolVal.Type == DSSymbolResponseValueType.Error)
                    {
                        // The symbol value is an error - perhaps there is no data
                        string errorMessage = symbolVal.Value.ToString();

                        Console.WriteLine("Service returned an error: " + errorMessage);
                    }
                    else if (symbolVal.Type == DSSymbolResponseValueType.DoubleArray)
                    {
                        // The symbol value is a double array
                        double[] values = (double[])symbolVal.Value;

                        Console.WriteLine("Values : ");
                        foreach (var val in values)
                        {
                            Console.WriteLine(val);
                        }
                    }
                }
              
            }
            
        }
                
    }
}
